property pSprite, pActive, alertflag, pAction, pVideoSprite

on getBehaviorDescription
  return "QT CONTROL BUTTON" & RETURN & RETURN & "Use this behavior to make almost any sprite into a control button for Quicktime video sprites. " & "Standard controls available through this behavior are Play, Stop/Pause, Rewind to beginning, Jump to End, Fast Forward (2x) and Fast backward (2x). " & "Drag behavior onto a sprite, select the Quicktime sprite to control, and which action the button will perform." & RETURN & RETURN & "PARAMETERS:" & RETURN & " - which Quicktime video sprite to control" & RETURN & " - Button action (rewind, stop, play, end, backward, forward)" & RETURN & RETURN & "PERMITTED TYPES:" & RETURN & " - Quicktime video to control" & RETURN & " - any graphics sprite(s) for the controls"
end

on getBehaviorTooltip
  return "Create custom QuickTime sprite video control buttons. " & "First place the QuickTime sprite on stage, then attach the behavior to the sprites that will serve as the controls."
end

on beginSprite me
  pAction = resolve(pAction)
  mInitialize(me)
end

on resolve prop
  case prop of
    pAction:
      choiceslist = ["Rewind", "Stop", "Play", "End", "Backward", "Forward"]
      lookup = [#rewind, #stop, #play, #end, #backward, #forward]
  end case
  return lookup[findPos(choiceslist, prop)]
end

on mouseDown me
  mVideoButton(me)
end

on mouseUp me
  case pAction of
    #backward, #forward:
      mVideoButton(me)
  end case
end

on mouseLeave me
  case pAction of
    #backward, #forward:
      mVideoButton(me)
  end case
end

on mInitialize me
  pSprite = sprite(me.spriteNum)
  pActive = 0
end

on mVideoButton me
  if mVerifyVideo(pVideoSprite) then
    case pAction of
      #play:
        sprite(pVideoSprite).movieRate = 1
      #stop:
        sprite(pVideoSprite).movieRate = 0
      #rewind:
        sprite(pVideoSprite).movieRate = 0
        sprite(pVideoSprite).movieTime = 0
      #end:
        sprite(pVideoSprite).movieRate = 0
        sprite(pVideoSprite).movieTime = sprite(pVideoSprite).member.duration
      #backward:
        if the mouseDown and rollover(me.spriteNum) then
          pActive = 1
          sprite(pVideoSprite).movieRate = -2
        else
          if pActive then
            sprite(pVideoSprite).movieRate = 0
          end if
          pActive = 0
        end if
      #forward:
        if the mouseDown and rollover(me.spriteNum) then
          pActive = 1
          sprite(pVideoSprite).movieRate = 2
        else
          if pActive then
            sprite(pVideoSprite).movieRate = 0
          end if
          pActive = 0
        end if
    end case
  end if
end

on mVerifyVideo vSpriteNum
  return sprite(vSpriteNum).member.type = #quickTimeMedia
end

on mVideoSpriteList
  vVideoList = []
  repeat with i = 1 to the lastChannel
    if mVerifyVideo(i) then
      add(vVideoList, i)
    end if
  end repeat
  return vVideoList
end

on mErrorAlert me
  if (the ticks - alertflag) > 10 then
    alert("No video sprites are currently on the Stage.")
    alertflag = the ticks
  end if
end

on isOKToAttach me, aSpriteType, aSpriteNum
  return aSpriteType = #graphic
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  vVideos = mVideoSpriteList()
  if not vVideos.count then
    return mErrorAlert(me)
  else
    vPDList = [:]
    setaProp(vPDList, #pVideoSprite, [#comment: "Video sprite channel", #format: #string, #default: vVideos[1], #range: vVideos])
    setaProp(vPDList, #pAction, [#comment: "Video button action", #format: #string, #default: "Play", #range: ["Rewind", "Stop", "Play", "End", "Backward", "Forward"]])
    return vPDList
  end if
end
